# !/usr/bin/bash
#total 4 functions, the main function is do_M5()

#main function
# $1: like "ipmitool -H host -U user -P pwd"
# $2: the dir you want to use for save the result
retry_times=2
function do_M5()
{
	if [ $1 ];then
		hostname=$1
	else
		echo "no hostname"
		return
	fi
	if [ $2 ];then
		username=$2
	else
		echo "no username"
		return
	fi
	if [ $3 ];then
		password=$3
	else
		echo "no password"
		return
	fi
	if [ $4 ];then
		logdir=$4
	else
		echo "no logdir"
		return
	fi
	ipmitool -V
	if [ $? -ne 0 ]
	then
		echo "check your ipmitool error"
		return
	fi
	commands="ipmitool -H$hostname -U$username -P$password -Ilanplus"
	filename_csr="$logdir/skl_csr.txt";
	#do_M5_MSR $commands $logdir
	#do_M5_CSR $commands $logdir
	do_M5_PRP $commands $logdir
}

function do_M5_MSR()
{
	filename_corenum="${logdir}/m5_corenum.txt"
	filename_msr="${logdir}/m5_msr.txt"
	if [ -f $filename_corenum ]
	then
		echo '' > $filename_corenum
	fi
	if [ -f $filename_msr ]
	then
		echo '' > $filename_msr
	fi

	cmd=" raw 0x32 0xBF 0x06 0x00 "
	cpunum=("0x30" "0x31")
	corenum=0
	for cpuid in ${cpunum[@]}
	do
		cmd_cpuid="$cmd $cpuid 0x00 0x00 0x08 0x00 "
		cmdGetCoreNum="$cmd_cpuid 0x00 0x53 0x00 "
		for retry in {1..$retry_times}
		do
		    result=`$commands $cmdGetCoreNum` &> /dev/null
		if [ "$result" != "" ] 
		then
            echo $result  > $filename_corenum
		    break
        else
            echo "get msr failed! Please check it, and try again."
            exit -1
		fi
		done
		cat $filename_corenum | grep "40"
		if [ $? -ne 0 ];then
			corenum=28
		else
			corenum=`cat $filename_corenum | awk -F'40 ' '{print $2}'`
		fi
		reg=0
		#i=0:CTL, i=1:STATUS, i=2:ADDR, I=3:MISC
        reg_names=("_CTL" "_STATUS" "_ADDR" "_MISC")
		for i in {0..3}
		do
			for coreid in $(seq 0 $corenum)
			do
				for left in {0..3}
				do
					let reg=i+left*4
					cmdReg="$cmd_cpuid 0x$(printf "%x" $coreid) "
					regNum="0x$(printf "%x" $reg) 0x04"
					regNum_print="0x4$(printf "%02x" $reg)"
					cmdReg="$cmdReg $regNum"
                    let reg_seq=reg/4
                    reg_name=$(printf "%s%d%s" "MC" $reg_seq ${reg_names[$i]})
					tips="cpu($(echo $cpuid|awk -F"0x3" '{print $2}')),coreid(0x$(printf "%x" $coreid)),regid($regNum_print),reg($reg_name):"
					for retry in {1..$retry_times}
					do
						result=`$commands $cmdReg` &> /dev/null
						if [ "$result" ] 
						then
							break
						fi
					done
					echo "$tips$result" >> $filename_msr
				done
			done
            let reg_init=16+i
			for left in {0..15}
			do
				let reg=reg_init+left*4
				cmdReg="$cmd_cpuid 0x00  0x$(printf "%x" $reg) 0x04"
                let reg_seq=reg/4
                reg_name=$(printf "%s%d%s" "MC" $reg_seq ${reg_names[$i]})
				tips="cpu($(echo $cpuid|awk -F"0x3" '{print $2}')),coreid(0x00),regid(0x4$(printf "%02x" $reg))),reg($reg_name):"
				for retry in {1..$retry_times}
				do
					result=`$commands $cmdReg` &> /dev/null
					if [ "$result" ] 
					then
						break
					fi
				done
				echo "$tips$result" >> $filename_msr
			done
			let i++
		done

		tips="cpu($(echo $cpuid|awk -F"0x3" '{print $2}')),coreid(0x00),regid(0x179),reg(IA32_MCG_CAP):"
		cmdReg="$cmd_cpuid 0x00 0x79 0x01"
		for retry in {1..$retry_times}
		do
			result=`$commands $cmdReg` &> /dev/null
			if [ "$result" ] 
			then
				break
			fi
		done
		echo "$tips$result" >> $filename_msr

		tips="cpu($(echo $cpuid|awk -F"0x3" '{print $2}')),coreid(0x00),regid(0x17C),reg(CORE_SMI_ERR_SRC):"
		cmdReg="$cmd_cpuid 0x00 0x7c 0x01"
		for retry in {1..$retry_times}
		do
			result=`$commands $cmdReg` &> /dev/null
			if [ "$result" ] 
			then
				break
			fi
		done
		echo "$tips$result" >> $filename_msr

		tips="cpu($(echo $cpuid|awk -F"0x3" '{print $2}')),coreid(0x00),regid(0X17E),reg(UNCORE_SMI_ERR_SRC):"
		cmdReg="$cmd_cpuid 0x00 0x7e 0x01"
		for retry in {1..$retry_times}
		do
			result=`$commands $cmdReg` &> /dev/null
			if [ "$result" ] 
			then
				break
			fi
		done
		echo "$tips$result" >> $filename_msr

		tips="cpu($(echo $cpuid|awk -F"0x3" '{print $2}')),coreid(0x00),regid(0X17F),reg(ERROR_CONTROL):"
		cmdReg="$cmd_cpuid 0x00 0x7f 0x01"
		for retry in {1..$retry_times}
		do
			result=`$commands $cmdReg` &> /dev/null
			if [ "$result" ] 
			then
				break
			fi
		done
		echo "$tips$result" >> $filename_msr

		tips="cpu($(echo $cpuid|awk -F"0x3" '{print $2}')),coreid(0x00),regid(0x52),reg(MCA_ERROR_CONTROL):"
		cmdReg="$cmd_cpuid 0x00 0x7f 0x01"
		for retry in {1..$retry_times}
		do
			result=`$commands $cmdReg` &> /dev/null
			if [ "$result" ] 
			then
				break
			fi
		done
		result=`$commands $cmdReg` &> /dev/null
		echo "$tips$result" >> $filename_msr
	done
}


function do_M5_CSR()
{
    csr_filename="config/csr_regAddr.txt"
    if [ ! -f $csr_filename ]
    then
        echo "there is no file named "$csr_filename
        exit -1
    fi

    while read line
    do
        regName=`echo $line |awk -F":" '{print $1}'`
        data=`echo $line |awk -F":" '{print $2}'`
        register=$(echo $data|awk -F"," '{print $4}'|sed  s/[[:space:]]//g)
        function=$(echo $data|awk -F"," '{print $3}'|sed  s/[[:space:]]//g)
        device=$(echo $data|awk -F"," '{print $2}'  |sed  s/[[:space:]]//g)
        bus=$(echo $data|awk -F"," '{print $1}'     |sed  s/[[:space:]]//g)
        if [ $(echo $register|wc -c) -eq 6 ] &> /dev/null
        then
            left="${register:0:2}${register:0-2:2}"
            left_extra=${register:2:1}
        elif [ $(echo $register|wc -c) -eq 5 ]
        then
            left=$register
            left_extra="0"
        else
            echo "cannot recognise the register."
            return 1
        fi
        let device_extra=device%2
        let device_value=device/2
        let middle=device_extra*8+function
        middle="0x"$(echo "obase=16;$middle"|bc)$left_extra
        right="0x"$(echo "obase=16;$bus"|bc)$(echo "obase=16;$device_value"|bc)
    	cmd="raw 0x32 0xBF 0x08 0x00 "
    	cpunum=("0x30" "0x31")
	    for cpuid in ${cpunum[@]}
    	do
    		cmdcpuid="$cmd $cpuid"
    		cmdcpuid="$cmdcpuid 0x00 0x00 0x01 0x04 0x00"
    		cmdGetCsr="$cmdcpuid $left $middle $right 0x00"
	    	tips="regName=$regName,cpu($(echo $cpuid|awk -F"0x3" '{print $2}')),coreid(0x00),csr($register):" &> /dev/null
		    for retry in {1..$retry_times}
            do
    		    result=`$commands $cmdGetCsr` &> /dev/null 
                if [ "$result" ]
                then
                    break
                fi
            done
	    	echo "$tips$result" >> $filename_csr
    	done
    done < $csr_filename
}


function do_M5_PRP()
{
	filename_prp="$logdir/m5_peci_root_port.txt"
	cmd="-t 0x2c -b 0x06 raw 0x2e 0x40 0x57 0x01 0x00 "
	cpunum=("0x30" "0x31")
	for cpuid in ${cpunum[@]}
	do
        if [ ! -f $filename_csr ]
        then
            echo "You should run do_M5_msr funciton first."
            exit -1
        fi
        #busnums=`cat $filename_csr |grep "CPUBUSNO " |grep "CPU${cpuid:0:1}"|awk -F"57 01 00 40" '{print $2}'|head -n1`
        busnums=`cat $filename_csr |grep "CPUBUSNO " |grep "CPU${cpuid:0-1:1}"|awk -F"57 01 00 40" '{print $2}'|head -n1`
        busnums=($busnums)
	    cmd_cpuid="$cmd $cpuid"
    	cmdcpuidprefix="$cmd_cpuid 0x06 0x05 0x61 0x00 "
		for busid in ${busnums[@]}
		do
			thirdPara=("0x${busid:1:1}0" "0x${busid:1:1}1")
			type=("0x4c 0x01" "0x58 0x01" "0x78 0x01" "0x4c 0x81" "0x58 0x81" "0x78 0x81")
            reg_name=("0x14c" "0x158" "0x178")
			cmdGetPceiRootPort_two="";
			for index in {0..5}
			do
                regType=${type[$index]}
				cmdGetPceiRootPort_two="$cmdcpuidprefix $regType"
				forthPara="0x0${busid:0:1}"
				for eachThirdPara in ${thirdPara[@]}
				do
					cmdGetPceiRootPort_four="$cmdGetPceiRootPort_two $eachThirdPara $forthPara"
                    let reg_name_index=index%3
                    device=0
                    if [ $index -le 2 ]
                    then
                        let device+=1
                    fi
                    echo $eachThirdPara|grep "1$" &> /dev/null
                    if [ $? -eq 0 ]
                    then
                        let device+=2
                    fi
					tips="cpu($(echo $cpuid|awk -F"0x3" '{print $2}')),rootbus($busid),device($device),register(0x$(printf "%x" ${reg_name[$reg_name_index]})):"
		            for retry in {1..$retry_times}
                    do
					    result=`$commands $cmdGetPceiRootPort_four` &> /dev/null
                        if [ "$result" ]
                        then
                            break
                        fi
                    done
					echo "$tips$result" >>  $filename_prp
				done
			done
		done
	done
}

do_M5 $1 $2 $3 $4
